/******************************************************************************
              Copyright 2000-2001 by O'ksi'D

Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of O'ksi'D
not be used in advertising or publicity pertaining to distribution
of the software without specific, written prior permission.
O'ksi'D makes no representations about the suitability of
this software for any purpose.  It is provided "as is" without
express or implied warranty.

O'ksi'D DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
IN NO EVENT SHALL O'ksi'D BE LIABLE FOR ANY SPECIAL, INDIRECT
OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
OR PERFORMANCE OF THIS SOFTWARE.

        Author: Jean-Marc Lienher ( http://oksid.ch )

******************************************************************************/

#include "headers/big5_0_.h"
#include "headers/dingbats_.h"
#include "headers/gb2312_1980_0_.h"
#include "headers/iso8859_10_.h"
#include "headers/iso8859_13_.h"
#include "headers/iso8859_14_.h"
#include "headers/iso8859_15_.h"
#include "headers/iso8859_1_.h"
#include "headers/iso8859_2_.h"
#include "headers/iso8859_3_.h"
#include "headers/iso8859_4_.h"
#include "headers/iso8859_5_.h"
#include "headers/iso8859_6_.h"
#include "headers/iso8859_7_.h"
#include "headers/iso8859_8_.h"
#include "headers/iso8859_9_.h"
#include "headers/jisx0201_1976_0_.h"
#include "headers/jisx0208_1983_0_.h"
#include "headers/jisx0212_1990_0_.h"
#include "headers/koi8_1_.h"
#include "headers/ksc5601_1987_0_.h"
#include "headers/symbol_.h"
  
   /*************** conv_gen.c ************/
const int ucs2fontmap(char *s, unsigned int ucs, int enc)
{
  switch(enc) {
  case 0:
    s[0] = (char) ((ucs & 0xFF00) >> 8);
    s[1] = (char) (ucs & 0xFF);
    return 0;
    break;
  case 1:
    if (ucs <= 0x00FF) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_1_1b_0001[ucs - 0x0001];
        if (s[1]) return 1;
      }
    }
    break;
  case 2:
    if (ucs <= 0x017E) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_2_1b_0001[ucs - 0x0001];
        if (s[1]) return 2;
      }
    } else if (ucs <= 0x02DD) {
      if (ucs >= 0x02C7) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_2_1b_02C7[ucs - 0x02C7];
        if (s[1]) return 2;
      }
    }
    break;
  case 3:
    if (ucs <= 0x017C) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_3_1b_0001[ucs - 0x0001];
        if (s[1]) return 3;
      }
    } else if (ucs <= 0x02D9) {
      if (ucs >= 0x02D8) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_3_1b_02D8[ucs - 0x02D8];
        if (s[1]) return 3;
      }
    }
    break;
  case 4:
    if (ucs <= 0x017E) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_4_1b_0001[ucs - 0x0001];
        if (s[1]) return 4;
      }
    } else if (ucs <= 0x02DB) {
      if (ucs >= 0x02C7) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_4_1b_02C7[ucs - 0x02C7];
        if (s[1]) return 4;
      }
    }
    break;
  case 5:
    if (ucs <= 0x00AD) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_5_1b_0001[ucs - 0x0001];
        if (s[1]) return 5;
      }
    } else if (ucs <= 0x045F) {
      if (ucs >= 0x0401) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_5_1b_0401[ucs - 0x0401];
        if (s[1]) return 5;
      }
    } else if (ucs <= 0x2116) {
      if (ucs >= 0x2116) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_5_1b_2116[ucs - 0x2116];
        if (s[1]) return 5;
      }
    }
    break;
  case 6:
    if (ucs <= 0x00AD) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_6_1b_0001[ucs - 0x0001];
        if (s[1]) return 6;
      }
    } else if (ucs <= 0x0652) {
      if (ucs >= 0x060C) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_6_1b_060C[ucs - 0x060C];
        if (s[1]) return 6;
      }
    }
    break;
  case 7:
    if (ucs <= 0x00BD) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_7_1b_0001[ucs - 0x0001];
        if (s[1]) return 7;
      }
    } else if (ucs <= 0x03CE) {
      if (ucs >= 0x0384) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_7_1b_0384[ucs - 0x0384];
        if (s[1]) return 7;
      }
    } else if (ucs <= 0x2019) {
      if (ucs >= 0x2015) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_7_1b_2015[ucs - 0x2015];
        if (s[1]) return 7;
      }
    }
    break;
  case 8:
    if (ucs <= 0x00F7) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_8_1b_0001[ucs - 0x0001];
        if (s[1]) return 8;
      }
    } else if (ucs <= 0x05EA) {
      if (ucs >= 0x05D0) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_8_1b_05D0[ucs - 0x05D0];
        if (s[1]) return 8;
      }
    } else if (ucs <= 0x2017) {
      if (ucs >= 0x200E) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_8_1b_200E[ucs - 0x200E];
        if (s[1]) return 8;
      }
    }
    break;
  case 9:
    if (ucs <= 0x015F) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_9_1b_0001[ucs - 0x0001];
        if (s[1]) return 9;
      }
    }
    break;
  case 10:
    if (ucs <= 0x017E) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_10_1b_0001[ucs - 0x0001];
        if (s[1]) return 10;
      }
    } else if (ucs <= 0x2015) {
      if (ucs >= 0x2015) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_10_1b_2015[ucs - 0x2015];
        if (s[1]) return 10;
      }
    }
    break;
  case 11:
    if (ucs <= 0x017E) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_13_1b_0001[ucs - 0x0001];
        if (s[1]) return 11;
      }
    } else if (ucs <= 0x201E) {
      if (ucs >= 0x2019) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_13_1b_2019[ucs - 0x2019];
        if (s[1]) return 11;
      }
    }
    break;
  case 12:
    if (ucs <= 0x0178) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_14_1b_0001[ucs - 0x0001];
        if (s[1]) return 12;
      }
    } else if (ucs <= 0x1EF3) {
      if (ucs >= 0x1E02) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_14_1b_1E02[ucs - 0x1E02];
        if (s[1]) return 12;
      }
    }
    break;
  case 13:
    if (ucs <= 0x017E) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_15_1b_0001[ucs - 0x0001];
        if (s[1]) return 13;
      }
    } else if (ucs <= 0x20AC) {
      if (ucs >= 0x20AC) {
        s[0] = 0;
        s[1] = unicode_to_iso8859_15_1b_20AC[ucs - 0x20AC];
        if (s[1]) return 13;
      }
    }
    break;
  case 14:
    if (ucs <= 0x00F7) {
      if (ucs >= 0x0001) {
        s[0] = 0;
        s[1] = unicode_to_koi8_1_1b_0001[ucs - 0x0001];
        if (s[1]) return 14;
      }
    } else if (ucs <= 0x0451) {
      if (ucs >= 0x0401) {
        s[0] = 0;
        s[1] = unicode_to_koi8_1_1b_0401[ucs - 0x0401];
        if (s[1]) return 14;
      }
    } else if (ucs <= 0x2265) {
      if (ucs >= 0x2219) {
        s[0] = 0;
        s[1] = unicode_to_koi8_1_1b_2219[ucs - 0x2219];
        if (s[1]) return 14;
      }
    } else if (ucs <= 0x2321) {
      if (ucs >= 0x2320) {
        s[0] = 0;
        s[1] = unicode_to_koi8_1_1b_2320[ucs - 0x2320];
        if (s[1]) return 14;
      }
    } else if (ucs <= 0x25A0) {
      if (ucs >= 0x2500) {
        s[0] = 0;
        s[1] = unicode_to_koi8_1_1b_2500[ucs - 0x2500];
        if (s[1]) return 14;
      }
    }
    break;
  case 15:
    if (ucs <= 0x00F7) {
      if (ucs >= 0x00A2) {
        int i = (ucs - 0x00A2) * 2;
        s[0] = unicode_to_big5_0_2b_00A2[i++];
        s[1] = unicode_to_big5_0_2b_00A2[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x02D9) {
      if (ucs >= 0x02C7) {
        int i = (ucs - 0x02C7) * 2;
        s[0] = unicode_to_big5_0_2b_02C7[i++];
        s[1] = unicode_to_big5_0_2b_02C7[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x0451) {
      if (ucs >= 0x0391) {
        int i = (ucs - 0x0391) * 2;
        s[0] = unicode_to_big5_0_2b_0391[i++];
        s[1] = unicode_to_big5_0_2b_0391[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x203E) {
      if (ucs >= 0x2013) {
        int i = (ucs - 0x2013) * 2;
        s[0] = unicode_to_big5_0_2b_2013[i++];
        s[1] = unicode_to_big5_0_2b_2013[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x2199) {
      if (ucs >= 0x2103) {
        int i = (ucs - 0x2103) * 2;
        s[0] = unicode_to_big5_0_2b_2103[i++];
        s[1] = unicode_to_big5_0_2b_2103[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x22BF) {
      if (ucs >= 0x221A) {
        int i = (ucs - 0x221A) * 2;
        s[0] = unicode_to_big5_0_2b_221A[i++];
        s[1] = unicode_to_big5_0_2b_221A[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x247D) {
      if (ucs >= 0x2460) {
        int i = (ucs - 0x2460) * 2;
        s[0] = unicode_to_big5_0_2b_2460[i++];
        s[1] = unicode_to_big5_0_2b_2460[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x2642) {
      if (ucs >= 0x2500) {
        int i = (ucs - 0x2500) * 2;
        s[0] = unicode_to_big5_0_2b_2500[i++];
        s[1] = unicode_to_big5_0_2b_2500[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x3129) {
      if (ucs >= 0x3000) {
        int i = (ucs - 0x3000) * 2;
        s[0] = unicode_to_big5_0_2b_3000[i++];
        s[1] = unicode_to_big5_0_2b_3000[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x32A3) {
      if (ucs >= 0x32A3) {
        int i = (ucs - 0x32A3) * 2;
        s[0] = unicode_to_big5_0_2b_32A3[i++];
        s[1] = unicode_to_big5_0_2b_32A3[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x33D5) {
      if (ucs >= 0x338E) {
        int i = (ucs - 0x338E) * 2;
        s[0] = unicode_to_big5_0_2b_338E[i++];
        s[1] = unicode_to_big5_0_2b_338E[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x7E9C) {
      if (ucs >= 0x4E00) {
        int i = (ucs - 0x4E00) * 2;
        s[0] = unicode_to_big5_0_2b_4E00[i++];
        s[1] = unicode_to_big5_0_2b_4E00[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x8B9F) {
      if (ucs >= 0x7F36) {
        int i = (ucs - 0x7F36) * 2;
        s[0] = unicode_to_big5_0_2b_7F36[i++];
        s[1] = unicode_to_big5_0_2b_7F36[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x9483) {
      if (ucs >= 0x8C37) {
        int i = (ucs - 0x8C37) * 2;
        s[0] = unicode_to_big5_0_2b_8C37[i++];
        s[1] = unicode_to_big5_0_2b_8C37[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0x9FA4) {
      if (ucs >= 0x9577) {
        int i = (ucs - 0x9577) * 2;
        s[0] = unicode_to_big5_0_2b_9577[i++];
        s[1] = unicode_to_big5_0_2b_9577[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0xFA0D) {
      if (ucs >= 0xFA0C) {
        int i = (ucs - 0xFA0C) * 2;
        s[0] = unicode_to_big5_0_2b_FA0C[i++];
        s[1] = unicode_to_big5_0_2b_FA0C[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0xFE6B) {
      if (ucs >= 0xFE30) {
        int i = (ucs - 0xFE30) * 2;
        s[0] = unicode_to_big5_0_2b_FE30[i++];
        s[1] = unicode_to_big5_0_2b_FE30[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0xFF64) {
      if (ucs >= 0xFF01) {
        int i = (ucs - 0xFF01) * 2;
        s[0] = unicode_to_big5_0_2b_FF01[i++];
        s[1] = unicode_to_big5_0_2b_FF01[i];
        if (s[0] || s[1]) return 15;
      }
    } else if (ucs <= 0xFFFD) {
      if (ucs >= 0xFFFD) {
        int i = (ucs - 0xFFFD) * 2;
        s[0] = unicode_to_big5_0_2b_FFFD[i++];
        s[1] = unicode_to_big5_0_2b_FFFD[i];
        if (s[0] || s[1]) return 15;
      }
    }
    break;
  case 16:
    if (ucs <= 0x0167) {
      if (ucs >= 0x00A1) {
        int i = (ucs - 0x00A1) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_00A1[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_00A1[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0x02DD) {
      if (ucs >= 0x02C7) {
        int i = (ucs - 0x02C7) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_02C7[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_02C7[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0x0451) {
      if (ucs >= 0x0391) {
        int i = (ucs - 0x0391) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_0391[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_0391[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0x2312) {
      if (ucs >= 0x2015) {
        int i = (ucs - 0x2015) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_2015[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_2015[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0x266D) {
      if (ucs >= 0x2460) {
        int i = (ucs - 0x2460) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_2460[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_2460[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0x327F) {
      if (ucs >= 0x3000) {
        int i = (ucs - 0x3000) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_3000[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_3000[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0x33DD) {
      if (ucs >= 0x3380) {
        int i = (ucs - 0x3380) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_3380[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_3380[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0x7E9C) {
      if (ucs >= 0x4E00) {
        int i = (ucs - 0x4E00) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_4E00[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_4E00[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0x8B9A) {
      if (ucs >= 0x7F36) {
        int i = (ucs - 0x7F36) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_7F36[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_7F36[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0x947F) {
      if (ucs >= 0x8C37) {
        int i = (ucs - 0x8C37) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_8C37[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_8C37[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0x9C57) {
      if (ucs >= 0x9577) {
        int i = (ucs - 0x9577) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_9577[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_9577[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0x9F9C) {
      if (ucs >= 0x9CE5) {
        int i = (ucs - 0x9CE5) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_9CE5[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_9CE5[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0xD79D) {
      if (ucs >= 0xAC00) {
        int i = (ucs - 0xAC00) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_AC00[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_AC00[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0xFA0B) {
      if (ucs >= 0xF900) {
        int i = (ucs - 0xF900) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_F900[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_F900[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0xFF5E) {
      if (ucs >= 0xFF01) {
        int i = (ucs - 0xFF01) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_FF01[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_FF01[i];
        if (s[0] || s[1]) return 16;
      }
    } else if (ucs <= 0xFFE6) {
      if (ucs >= 0xFFE0) {
        int i = (ucs - 0xFFE0) * 2;
        s[0] = unicode_to_ksc5601_1987_0_2b_FFE0[i++];
        s[1] = unicode_to_ksc5601_1987_0_2b_FFE0[i];
        if (s[0] || s[1]) return 16;
      }
    }
    break;
  case 17:
    if (ucs <= 0x01DC) {
      if (ucs >= 0x00A4) {
        int i = (ucs - 0x00A4) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_00A4[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_00A4[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x02C9) {
      if (ucs >= 0x02C7) {
        int i = (ucs - 0x02C7) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_02C7[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_02C7[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x0451) {
      if (ucs >= 0x0391) {
        int i = (ucs - 0x0391) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_0391[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_0391[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x203B) {
      if (ucs >= 0x2015) {
        int i = (ucs - 0x2015) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_2015[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_2015[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x2312) {
      if (ucs >= 0x2103) {
        int i = (ucs - 0x2103) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_2103[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_2103[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x2642) {
      if (ucs >= 0x2460) {
        int i = (ucs - 0x2460) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_2460[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_2460[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x3129) {
      if (ucs >= 0x3000) {
        int i = (ucs - 0x3000) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_3000[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_3000[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x3229) {
      if (ucs >= 0x3220) {
        int i = (ucs - 0x3220) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_3220[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_3220[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x7DAE) {
      if (ucs >= 0x4E00) {
        int i = (ucs - 0x4E00) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_4E00[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_4E00[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x8C98) {
      if (ucs >= 0x7E3B) {
        int i = (ucs - 0x7E3B) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_7E3B[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_7E3B[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x8ECE) {
      if (ucs >= 0x8D1D) {
        int i = (ucs - 0x8D1D) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_8D1D[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_8D1D[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x91DC) {
      if (ucs >= 0x8F66) {
        int i = (ucs - 0x8F66) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_8F66[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_8F66[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x99A8) {
      if (ucs >= 0x9274) {
        int i = (ucs - 0x9274) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_9274[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_9274[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x9B54) {
      if (ucs >= 0x9A6C) {
        int i = (ucs - 0x9A6C) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_9A6C[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_9A6C[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x9CE2) {
      if (ucs >= 0x9C7C) {
        int i = (ucs - 0x9C7C) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_9C7C[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_9C7C[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0x9FA0) {
      if (ucs >= 0x9E1F) {
        int i = (ucs - 0x9E1F) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_9E1F[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_9E1F[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0xFF5E) {
      if (ucs >= 0xFF01) {
        int i = (ucs - 0xFF01) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_FF01[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_FF01[i];
        if (s[0] || s[1]) return 17;
      }
    } else if (ucs <= 0xFFE5) {
      if (ucs >= 0xFFE0) {
        int i = (ucs - 0xFFE0) * 2;
        s[0] = unicode_to_gb2312_1980_0_2b_FFE0[i++];
        s[1] = unicode_to_gb2312_1980_0_2b_FFE0[i];
        if (s[0] || s[1]) return 17;
      }
    }
    break;
  case 18:
    if (ucs <= 0x00A5) {
      if (ucs >= 0x0020) {
        s[0] = 0;
        s[1] = unicode_to_jisx0201_1976_0_1b_0020[ucs - 0x0020];
        if (s[1]) return 18;
      }
    } else if (ucs <= 0x203E) {
      if (ucs >= 0x203E) {
        s[0] = 0;
        s[1] = unicode_to_jisx0201_1976_0_1b_203E[ucs - 0x203E];
        if (s[1]) return 18;
      }
    } else if (ucs <= 0xFF9F) {
      if (ucs >= 0xFF61) {
        s[0] = 0;
        s[1] = unicode_to_jisx0201_1976_0_1b_FF61[ucs - 0xFF61];
        if (s[1]) return 18;
      }
    }
    break;
  case 19:
    if (ucs <= 0x00F7) {
      if (ucs >= 0x005C) {
        int i = (ucs - 0x005C) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_005C[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_005C[i];
        if (s[0] || s[1]) return 19;
      }
    } else if (ucs <= 0x0451) {
      if (ucs >= 0x0391) {
        int i = (ucs - 0x0391) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_0391[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_0391[i];
        if (s[0] || s[1]) return 19;
      }
    } else if (ucs <= 0x203B) {
      if (ucs >= 0x2010) {
        int i = (ucs - 0x2010) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_2010[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_2010[i];
        if (s[0] || s[1]) return 19;
      }
    } else if (ucs <= 0x2312) {
      if (ucs >= 0x2103) {
        int i = (ucs - 0x2103) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_2103[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_2103[i];
        if (s[0] || s[1]) return 19;
      }
    } else if (ucs <= 0x266F) {
      if (ucs >= 0x2500) {
        int i = (ucs - 0x2500) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_2500[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_2500[i];
        if (s[0] || s[1]) return 19;
      }
    } else if (ucs <= 0x30FE) {
      if (ucs >= 0x3000) {
        int i = (ucs - 0x3000) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_3000[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_3000[i];
        if (s[0] || s[1]) return 19;
      }
    } else if (ucs <= 0x7E9C) {
      if (ucs >= 0x4E00) {
        int i = (ucs - 0x4E00) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_4E00[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_4E00[i];
        if (s[0] || s[1]) return 19;
      }
    } else if (ucs <= 0x8B9A) {
      if (ucs >= 0x7F36) {
        int i = (ucs - 0x7F36) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_7F36[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_7F36[i];
        if (s[0] || s[1]) return 19;
      }
    } else if (ucs <= 0x9481) {
      if (ucs >= 0x8C37) {
        int i = (ucs - 0x8C37) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_8C37[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_8C37[i];
        if (s[0] || s[1]) return 19;
      }
    } else if (ucs <= 0x9FA0) {
      if (ucs >= 0x9577) {
        int i = (ucs - 0x9577) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_9577[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_9577[i];
        if (s[0] || s[1]) return 19;
      }
    } else if (ucs <= 0xFF5D) {
      if (ucs >= 0xFF01) {
        int i = (ucs - 0xFF01) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_FF01[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_FF01[i];
        if (s[0] || s[1]) return 19;
      }
    } else if (ucs <= 0xFFE5) {
      if (ucs >= 0xFFE3) {
        int i = (ucs - 0xFFE3) * 2;
        s[0] = unicode_to_jisx0208_1983_0_2b_FFE3[i++];
        s[1] = unicode_to_jisx0208_1983_0_2b_FFE3[i];
        if (s[0] || s[1]) return 19;
      }
    }
    break;
  case 20:
    if (ucs <= 0x01F5) {
      if (ucs >= 0x007E) {
        int i = (ucs - 0x007E) * 2;
        s[0] = unicode_to_jisx0212_1990_0_2b_007E[i++];
        s[1] = unicode_to_jisx0212_1990_0_2b_007E[i];
        if (s[0] || s[1]) return 20;
      }
    } else if (ucs <= 0x02DD) {
      if (ucs >= 0x02C7) {
        int i = (ucs - 0x02C7) * 2;
        s[0] = unicode_to_jisx0212_1990_0_2b_02C7[i++];
        s[1] = unicode_to_jisx0212_1990_0_2b_02C7[i];
        if (s[0] || s[1]) return 20;
      }
    } else if (ucs <= 0x045F) {
      if (ucs >= 0x0384) {
        int i = (ucs - 0x0384) * 2;
        s[0] = unicode_to_jisx0212_1990_0_2b_0384[i++];
        s[1] = unicode_to_jisx0212_1990_0_2b_0384[i];
        if (s[0] || s[1]) return 20;
      }
    } else if (ucs <= 0x2122) {
      if (ucs >= 0x2116) {
        int i = (ucs - 0x2116) * 2;
        s[0] = unicode_to_jisx0212_1990_0_2b_2116[i++];
        s[1] = unicode_to_jisx0212_1990_0_2b_2116[i];
        if (s[0] || s[1]) return 20;
      }
    } else if (ucs <= 0x7E9E) {
      if (ucs >= 0x4E02) {
        int i = (ucs - 0x4E02) * 2;
        s[0] = unicode_to_jisx0212_1990_0_2b_4E02[i++];
        s[1] = unicode_to_jisx0212_1990_0_2b_4E02[i];
        if (s[0] || s[1]) return 20;
      }
    } else if (ucs <= 0x8B9F) {
      if (ucs >= 0x7F3B) {
        int i = (ucs - 0x7F3B) * 2;
        s[0] = unicode_to_jisx0212_1990_0_2b_7F3B[i++];
        s[1] = unicode_to_jisx0212_1990_0_2b_7F3B[i];
        if (s[0] || s[1]) return 20;
      }
    } else if (ucs <= 0x9484) {
      if (ucs >= 0x8C38) {
        int i = (ucs - 0x8C38) * 2;
        s[0] = unicode_to_jisx0212_1990_0_2b_8C38[i++];
        s[1] = unicode_to_jisx0212_1990_0_2b_8C38[i];
        if (s[0] || s[1]) return 20;
      }
    } else if (ucs <= 0x9FA5) {
      if (ucs >= 0x9578) {
        int i = (ucs - 0x9578) * 2;
        s[0] = unicode_to_jisx0212_1990_0_2b_9578[i++];
        s[1] = unicode_to_jisx0212_1990_0_2b_9578[i];
        if (s[0] || s[1]) return 20;
      }
    }
    break;
  case 21:
    if (ucs <= 0x00F7) {
      if (ucs >= 0x0020) {
        s[0] = 0;
        s[1] = unicode_to_symbol_1b_0020[ucs - 0x0020];
        if (s[1]) return 21;
      }
    } else if (ucs <= 0x0192) {
      if (ucs >= 0x0192) {
        s[0] = 0;
        s[1] = unicode_to_symbol_1b_0192[ucs - 0x0192];
        if (s[1]) return 21;
      }
    } else if (ucs <= 0x03D6) {
      if (ucs >= 0x0391) {
        s[0] = 0;
        s[1] = unicode_to_symbol_1b_0391[ucs - 0x0391];
        if (s[1]) return 21;
      }
    } else if (ucs <= 0x232A) {
      if (ucs >= 0x2022) {
        s[0] = 0;
        s[1] = unicode_to_symbol_1b_2022[ucs - 0x2022];
        if (s[1]) return 21;
      }
    } else if (ucs <= 0x25CA) {
      if (ucs >= 0x25CA) {
        s[0] = 0;
        s[1] = unicode_to_symbol_1b_25CA[ucs - 0x25CA];
        if (s[1]) return 21;
      }
    } else if (ucs <= 0x2666) {
      if (ucs >= 0x2660) {
        s[0] = 0;
        s[1] = unicode_to_symbol_1b_2660[ucs - 0x2660];
        if (s[1]) return 21;
      }
    } else if (ucs <= 0xF6DB) {
      if (ucs >= 0xF6D9) {
        s[0] = 0;
        s[1] = unicode_to_symbol_1b_F6D9[ucs - 0xF6D9];
        if (s[1]) return 21;
      }
    } else if (ucs <= 0xF8FE) {
      if (ucs >= 0xF8E5) {
        s[0] = 0;
        s[1] = unicode_to_symbol_1b_F8E5[ucs - 0xF8E5];
        if (s[1]) return 21;
      }
    }
    break;
  case 22:
    if (ucs <= 0x00A0) {
      if (ucs >= 0x0020) {
        s[0] = 0;
        s[1] = unicode_to_dingbats_1b_0020[ucs - 0x0020];
        if (s[1]) return 22;
      }
    } else if (ucs <= 0x2195) {
      if (ucs >= 0x2192) {
        s[0] = 0;
        s[1] = unicode_to_dingbats_1b_2192[ucs - 0x2192];
        if (s[1]) return 22;
      }
    } else if (ucs <= 0x2469) {
      if (ucs >= 0x2460) {
        s[0] = 0;
        s[1] = unicode_to_dingbats_1b_2460[ucs - 0x2460];
        if (s[1]) return 22;
      }
    } else if (ucs <= 0x2666) {
      if (ucs >= 0x25A0) {
        s[0] = 0;
        s[1] = unicode_to_dingbats_1b_25A0[ucs - 0x25A0];
        if (s[1]) return 22;
      }
    } else if (ucs <= 0x27BE) {
      if (ucs >= 0x2701) {
        s[0] = 0;
        s[1] = unicode_to_dingbats_1b_2701[ucs - 0x2701];
        if (s[1]) return 22;
      }
    } else if (ucs <= 0xF8E4) {
      if (ucs >= 0xF8D7) {
        s[0] = 0;
        s[1] = unicode_to_dingbats_1b_F8D7[ucs - 0xF8D7];
        if (s[1]) return 22;
      }
    }
    break;

  default:
    break;
  };
  return -1;
};

const int encoding_number(const char *enc)
{
  if (!enc || !strcmp(enc, "iso10646-1")) {
    return 0;
  } else if (!strcmp(enc, "iso8859-1")) {
    return 1;
  } else if (!strcmp(enc, "iso8859-2")) {
    return 2;
  } else if (!strcmp(enc, "iso8859-3")) {
    return 3;
  } else if (!strcmp(enc, "iso8859-4")) {
    return 4;
  } else if (!strcmp(enc, "iso8859-5")) {
    return 5;
  } else if (!strcmp(enc, "iso8859-6")) {
    return 6;
  } else if (!strcmp(enc, "iso8859-7")) {
    return 7;
  } else if (!strcmp(enc, "iso8859-8")) {
    return 8;
  } else if (!strcmp(enc, "iso8859-9")) {
    return 9;
  } else if (!strcmp(enc, "iso8859-10")) {
    return 10;
  } else if (!strcmp(enc, "iso8859-13")) {
    return 11;
  } else if (!strcmp(enc, "iso8859-14")) {
    return 12;
  } else if (!strcmp(enc, "iso8859-15")) {
    return 13;
  } else if (!strcmp(enc, "koi8-1")) {
    return 14;
  } else if (!strcmp(enc, "big5-0") || !strcmp(enc, "big5.eten-0")) {
    return 15;
  } else if (!strcmp(enc, "ksc5601.1987-0")) {
    return 16;
  } else if (!strcmp(enc, "gb2312.1980-0")) {
    return 17;
  } else if (!strcmp(enc, "jisx0201.1976-0")) {
    return 18;
  } else if (!strcmp(enc, "jisx0208.1983-0") || !strcmp(enc, "jisx0208.1990-0")) {
    return 19;
  } else if (!strcmp(enc, "jisx0212.1990-0")) {
    return 20;
  } else if (!strcmp(enc, "symbol")) {
    return 21;
  } else if (!strcmp(enc, "dingbats") || !strcmp(enc, "zapfdingbats") || !strcmp(enc, "zapf dingbats") || !strcmp(enc, "itc zapf dingbats")) {
    return 22;
  };
  return -1;
};

/*
const char *encoding_name(int num)
{
  switch (num) {
  case 1:
    return "iso8859-1";
  case 2:
    return "iso8859-2";
  case 3:
    return "iso8859-3";
  case 4:
    return "iso8859-4";
  case 5:
    return "iso8859-5";
  case 6:
    return "iso8859-6";
  case 7:
    return "iso8859-7";
  case 8:
    return "iso8859-8";
  case 9:
    return "iso8859-9";
  case 10:
    return "iso8859-10";
  case 11:
    return "iso8859-13";
  case 12:
    return "iso8859-14";
  case 13:
    return "iso8859-15";
  case 14:
    return "koi8-1";
  case 15:
    return "big5-0";
  case 16:
    return "ksc5601.1987-0";
  case 17:
    return "gb2312.1980-0";
  case 18:
    return "jisx0201.1976-0";
  case 19:
    return "jisx0208.1983-0";
  case 20:
    return "jisx0212.1990-0";
  case 21:
    return "symbol";
  case 22:
    return "dingbats";
  };
  return "iso10646-1";
};

*/
