/******************************************************************************
              Copyright (c) 2000-2001 by O'ksi'D

Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear
in supporting documentation, and that the name of O'ksi'D
not be used in advertising or publicity pertaining to distribution
of the software without specific, written prior permission.
O'ksi'D makes no representations about the suitability of
this software for any purpose.  It is provided "as is" without
express or implied warranty.

O'ksi'D DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS,
IN NO EVENT SHALL O'ksi'D BE LIABLE FOR ANY SPECIAL, INDIRECT
OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
OR PERFORMANCE OF THIS SOFTWARE.

******************************************************************************/

#include "Xutf8.h"
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Intrinsic.h>

/*
  I haven't much doc on the web about EUC encodings, so I've used
  GNU libiconv source code as a refrence.
  http://clisp.cons.org/~haible/packages-libiconv.html
*/

extern const int encoding_number(const char *enc);
extern const int ucs2fontmap(char *s, unsigned int ucs, int enc);

unsigned short *
XInitJisx212ToUcs(
	void)
{
	unsigned int i, wc;
	unsigned char c, c1;
	unsigned short * ret;
	int enc;
	char buf[2];

	ret = (unsigned short*) malloc(sizeof(unsigned short) * 
		(0x6D63 - 0x222F + 1));
	i = 0x6D63 - 0x222F + 1;
	while (i > 0) {
		i--;
		ret[i] = '?';
	}
	enc = encoding_number("jisx0212.1990-0");
	
	i = 0x10000;
	while (i > 0) {
		i--;
		buf[0] = 0;
		buf[1] = 0;
		ucs2fontmap(buf, i, enc);			
		c = (unsigned char) buf[0];		
		c1 = (unsigned char) buf[1];		
		wc = (c << 8) + c1;
		if (wc >= 0x222F && wc <= 0x6D63) {
			wc -= 0x222F;
			ret[wc] = i;
		}
	}
	return ret;
}

unsigned short *
XInitJisx201ToUcs(
	void)
{
	unsigned int i, wc;
	unsigned char c, c1;
	unsigned short * ret;
	int enc;
	char buf[2];

	ret = (unsigned short*) malloc(sizeof(unsigned short) * 
		(0xDF - 0x20 + 1));
	i = 0xDF - 0x20 + 1;
	while (i > 0) {
		i--;
		ret[i] = '?';
	}
	enc = encoding_number("jisx0201.1976-0");
	
	i = 0x10000;
	while (i > 0) {
		i--;
		buf[0] = 0;
		buf[1] = 0;
		ucs2fontmap(buf, i, enc);			
		c = (unsigned char) buf[0];		
		c1 = (unsigned char) buf[1];		
		wc = (c << 8) + c1;
		if (wc >= 0x20 && wc <= 0xDF) {
			wc -= 0x20;
			ret[wc] = i;
		}
	}
	return ret;
}


unsigned short *
XInitJisx208ToUcs(
	void)
{
	unsigned int i, wc;
	unsigned char c, c1;
	unsigned short * ret;
	int enc;
	char buf[2];

	ret = (unsigned short*) malloc(sizeof(unsigned short) * 
		(0x6D63 - 0x222F + 1));
	i = 0x6D63 - 0x222F + 1;
	while (i > 0) {
		i--;
		ret[i] = '?';
	}
	enc = encoding_number("jisx0208.1983-0");
	
	i = 0x10000;
	while (i > 0) {
		i--;
		buf[0] = 0;
		buf[1] = 0;
		ucs2fontmap(buf, i, enc);			
		c = (unsigned char) buf[0];		
		c1 = (unsigned char) buf[1];		
		wc = (c << 8) + c1;
		if (wc >= 0x222F && wc <= 0x6D63) {
			wc -= 0x222F;
			ret[wc] = i;
		}
	}
	return ret;
}

unsigned short *
XInitKsc5601ToUcs(
	void)
{
	unsigned int i, wc;
	unsigned char c, c1;
	unsigned short * ret;
	int enc;
	char buf[2];

	ret = (unsigned short*) malloc(sizeof(unsigned short) * 
		(0x7D7E - 0x2121 + 1));
	i = 0x7D7E - 0x2121 + 1;
	while (i > 0) {
		i--;
		ret[i] = '?';
	}
	enc = encoding_number("ksc5601.1987-0");
	
	i = 0x10000;
	while (i > 0) {
		i--;
		buf[0] = 0;
		buf[1] = 0;
		ucs2fontmap(buf, i, enc);			
		c = (unsigned char) buf[0];		
		c1 = (unsigned char) buf[1];		
		wc = (c << 8) + c1;
		if (wc >= 0x2121 && wc <= 0x7D7E) {
			wc -= 0x2121;
			ret[wc] = i;
		}
	}
	return ret;
}

unsigned short *
XInitBig5ToUcs(
	void)
{
	unsigned int i, wc;
	unsigned char c, c1;
	unsigned short * ret;
	int enc;
	char buf[2];

	ret = (unsigned short*) malloc(sizeof(unsigned short) * 
		(0xC947 - 0xA140 + 1));
	i = 0xC947 - 0xA140 + 1;
	while (i > 0) {
		i--;
		ret[i] = '?';
	}
	enc = encoding_number("big5-0");
	
	i = 0x10000;
	while (i > 0) {
		i--;
		buf[0] = 0;
		buf[1] = 0;
		ucs2fontmap(buf, i, enc);			
		c = (unsigned char) buf[0];		
		c1 = (unsigned char) buf[1];		
		wc = (c << 8) + c1;
		if (wc >= 0xA140 && wc <= 0xC947) {
			wc -= 0xA140;
			ret[wc] = i;
		}
	}
	return ret;
}


unsigned short *
XInitGb2312ToUcs(
	void)
{
	unsigned int i, wc;
	unsigned char c, c1;
	unsigned short * ret;
	int enc;
	char buf[2];

	ret = (unsigned short*) malloc(sizeof(unsigned short) * 
		(0x777E - 0x2121 + 1));
	i = 0x777E - 0x2121 + 1;
	while (i > 0) {
		i--;
		ret[i] = '?';
	}
	enc = encoding_number("gb2312.1980-0");
	
	i = 0x10000;
	while (i > 0) {
		i--;
		buf[0] = 0;
		buf[1] = 0;
		ucs2fontmap(buf, i, enc);			
		c = (unsigned char) buf[0];		
		c1 = (unsigned char) buf[1];		
		wc = (c << 8) + c1;
		if (wc >= 0x2121 && wc <= 0x777E) {
			wc -= 0x2121;
			ret[wc] = i;
		}
	}
	return ret;
}

int 
XConvertEucTwToUtf8(
	char*		buffer_return,
	int		len)
{
 /* FIXME */
	return len;
}

int 
XConvertEucKrToUtf8(
	char*		buffer_return,
	int		len)
{
	int i = 0, l = 0;
	char *buf;

	buf = (char*) malloc(len);
	memcpy(buf, buffer_return, len);

	while (i < len) {
		unsigned int ucs;
		unsigned char c, c1;
		unsigned int wc;
		c = (unsigned char) buf[i];
		if (c < 0x80) {
			ucs = c;	
			i++;
		} else if (c >= 0xA1 && c < 0xFF && len - i > 1) {
			c1 = (unsigned char) buf[i + 1];		
			wc = ((c - 0x80) << 8) + (c1 - 0x80);
			if (wc >= 0x8141 && wc <= 0x7D7E) {
				static unsigned short *ksc5601 = NULL;
				wc -= 0x2121;
				if (ksc5601 == NULL) {
					ksc5601 = XInitKsc5601ToUcs();
				}
				ucs = ksc5601[wc];
			} else {
				ucs = '?';
			}
			i += 2;
		} else {
			ucs = '?';
			i++;
		}
		l += XConvertUcsToUtf8(ucs, buffer_return + l);
	}
	free(buf);
	return l;
}

int 
XConvertBig5ToUtf8(
	char*		buffer_return,
	int		len)
{
	int i = 0, l = 0;
	char *buf;

	buf = (char*) malloc(len);
	memcpy(buf, buffer_return, len);

	while (i < len) {
		unsigned int ucs;
		unsigned char c, c1;
		unsigned int wc;
		c = (unsigned char) buf[i];
		if (len - i > 1) {
			c1 = (unsigned char) buf[i + 1];		
			wc = (c << 8) + c1;
			if (wc >= 0xA140 && wc <= 0xC947) {
				static unsigned short *big5 = NULL;
				wc -= 0xA140;
				if (big5 == NULL) {
					big5 = XInitBig5ToUcs();
				}
				ucs = big5[wc];
			} else {
				ucs = '?';
			}
			i += 2;
		} else {
			ucs = '?';
			i++;
		}
		l += XConvertUcsToUtf8(ucs, buffer_return + l);
	}
	free(buf);
	return l;
}

int 
XConvertEucCnToUtf8(
	char*		buffer_return,
	int		len)
{
	int i = 0, l = 0;
	char *buf;

	buf = (char*) malloc(len);
	memcpy(buf, buffer_return, len);

	while (i < len) {
		unsigned int ucs;
		unsigned char c, c1;
		unsigned int wc;
		c = (unsigned char) buf[i];
		if (c < 0x80) {
			ucs = c;	
			i++;
		} else if (c >= 0xA1 && c < 0xFF && len - i > 1) {
			c1 = (unsigned char) buf[i + 1];		
			wc = ((c - 0x80) << 8) + (c1 - 0x80);
			if (wc >= 0x2121 && wc <= 0x777E) {
				static unsigned short *gb2312 = NULL;
				wc -= 0x2121;
				if (gb2312 == NULL) {
					gb2312 = XInitGb2312ToUcs();
				}
				ucs = gb2312[wc];
			} else {
				ucs = '?';
			}
			i += 2;
		} else {
			ucs = '?';
			i++;
		}
		l += XConvertUcsToUtf8(ucs, buffer_return + l);
	}
	free(buf);
	return l;
}

int 
XConvertEucJpToUtf8(
	char*		buffer_return,
	int		len)
{
	int i = 0, l = 0;
	char *buf;

	buf = (char*) malloc(len);
	memcpy(buf, buffer_return, len);

	while (i < len) {
		unsigned int ucs;
		unsigned char c, c1;
		unsigned int wc;
		c = (unsigned char) buf[i];
		if (c < 0x80) {
			ucs = c;	
			i++;
		} else if (c >= 0xA1 && c < 0xFF && len - i > 1) {
			c1 = (unsigned char) buf[i + 1];		
			wc = ((c - 0x80) << 8) + (c1 - 0x80);
			if (c < 0xF5 && wc >= 0x222F && wc <= 0x6D63) {
				static unsigned short *jisx208 = NULL;
				wc -= 0x222F;
				if (jisx208 == NULL) {
					jisx208 = XInitJisx208ToUcs();
				}
				ucs = jisx208[wc];
			} else if (c1 >= 0xA1 && c1 < 0xFF) {
				ucs = 0xE000 + 94 * (c - 0xF5) + (c1 - 0xA1);
			} else {
				ucs = '?';
			}
			i += 2;
		} else if (c == 0x8E && len - i > 1) {
			static unsigned short *jisx201 = NULL;
			c1 = (unsigned char) buf[i + 1];		
			if (c1 >= 0x20 && c1 <= 0xDF) {
				c1 -= 0x20;
				if (jisx201 == NULL) {
					jisx201 = XInitJisx201ToUcs();
				}
				ucs = jisx201[c1];
			} else {
				ucs = '?';
			}
			i += 2;
		} else if (c == 0x8F && len - i > 2) {
			c = (unsigned char) buf[i + 1];		
			c1 = (unsigned char) buf[i + 2];		
			wc = ((c - 0x80) << 8) + (c1 - 0x80);
			if (c < 0xF5 && wc >= 0x222F && wc <= 0x6D63) {
				static unsigned short *jisx212 = NULL;
				wc -= 0x222F;
				if (jisx212 == NULL) {
					jisx212 = XInitJisx212ToUcs();
				}
				ucs = jisx212[wc];
			} else if (c1 >= 0xA1 && c1 < 0xFF) {
				ucs = 0xE000 + 94 * (c - 0xF5) + (c1 - 0xA1);
			} else {
				ucs = '?';
			}
			i += 3;
		} else {
			ucs = '?';
			i++;
		}
		l += XConvertUcsToUtf8(ucs, buffer_return + l);
	}
	free(buf);
	return l;
}

int
XConvertEucToUtf8(
	const char*	locale,
	char*		buffer_return, 
	int		len, 
	int		bytes_buffer)
{
	if (!locale || strstr(locale, "UTF") || strstr(locale, "utf")) {
		return len;
	}

	if (strstr(locale, "JP")) {	
		return XConvertEucJpToUtf8(buffer_return, len);
	} else if (strstr(locale, "Big5") || strstr(locale, "big5")) { // BIG5
		return XConvertBig5ToUtf8(buffer_return, len);
	} else if (strstr(locale, "CN")) { 
		return XConvertEucCnToUtf8(buffer_return, len);
	} else if (strstr(locale, "KR")) { 
		return XConvertEucKrToUtf8(buffer_return, len);
	} else if (strstr(locale, "TW")) { 
		return XConvertEucTwToUtf8(buffer_return, len);
	}
	return len;
}

int
XUtf8LookupString(
    XIC                 ic,
    XKeyPressedEvent*   event,
    char*               buffer_return,
    int                 bytes_buffer,
    KeySym*             keysym,
    Status*             status_return)
{
        long ucs = -1;
        int len;
        len = XmbLookupString(ic, event, buffer_return, bytes_buffer / 2,
                         keysym, status_return);
        if (len == 1) {
                /* Take ISO 8859-1 character delivered by XLookupString() */
                ucs = (unsigned char) buffer_return[0];
        } else if (!len && ((*keysym >= 0x100 && *keysym <= 0xf000) ||
                              (*keysym & 0xff000000U) == 0x01000000))
        {
                ucs = XKeysymToUcs(*keysym);
        } else {
                ucs = -2;
        }
        if (ucs == -1) len = 0;
        if (ucs >= 0) {
                len = XConvertUcsToUtf8(ucs, (char *)buffer_return);
        } else if (len > 0) {
		len = XConvertEucToUtf8(XLocaleOfIM(XIMOfIC(ic)), 
			buffer_return, len, bytes_buffer);	
	}
        return len;
}


